#define IN_LIBXML
#include <libxml/wrappers.h>

/**
 * xmlFindNodeList:
 * @cur:  first node pointer
 * @name: node name
 *
 * Returns node pointer or NULL.
 */
xmlNodePtr xmlFindNodeList (xmlNodePtr cur, const xmlChar *name) {
  if (cur == NULL || name == NULL)
    return (NULL);

  for( ; cur != NULL; cur = cur->next) {
    if(xmlStrEqual(cur->name, name))
      return cur;
    }
  return (NULL);
  }

/**
 * xmlFindNode:
 * @cur:  parent node pointer
 * @name: node name
 *
 * Returns child node pointer or NULL.
 */
xmlNodePtr xmlFindNode (xmlNodePtr cur, const xmlChar *name) {
  if (cur == NULL || name == NULL)
    return (NULL);

  for(cur = cur->children; cur != NULL; cur = cur->next) {
    if(xmlStrEqual(cur->name, name))
      return cur;
    }
  return (NULL);
  }

/**
 * xmlDocAddChild:
 * @doc:  the document pointer
 * @node: the node pointer
 *
 * Add @node to the end of document children list.
 *
 * Returns node pointer.
 */
xmlNodePtr xmlDocAddChild (xmlDocPtr doc, xmlNodePtr node) {
  if (doc  == NULL) return (NULL);
  if (node == NULL) return (NULL);

  xmlUnlinkNode(node);
  xmlSetTreeDoc(node, doc);
  node->parent = (xmlNodePtr) doc;

  if (doc->children != NULL) {
    xmlAddSibling(doc->children, node);
  } else {
    doc->children = node;
    doc->last = node;
    }
  return node;
  }

/**
 * xmlDocGetFirst:
 * @doc: the document pointer
 *
 * Returns pointer to first child node or NULL.
 */
xmlNodePtr xmlDocGetFirst (xmlDocPtr doc) {
  if (doc == NULL) return (NULL);
  return doc->children;
  }

/**
 * xmlDocStandalone:
 * @doc: the document pointer
 *
 * Returns standalone flag for document or 0.
 */
int xmlDocStandalone (xmlDocPtr doc) {
  if (doc == NULL) return 0;
  return doc->standalone;
  }

/**
 * xmlGetNodeType:
 * @cur: the node pointer
 *
 * Returns node type or 0 if attribute pointer is NULL.
 */
xmlElementType xmlGetNodeType (xmlNodePtr cur) {
  if (cur == NULL) return 0;
  return cur->type;
  }

/**
 * xmlGetNodeName:
 * @cur: the node pointer
 *
 * Returns pointer to node name or NULL.
 * The caller must not free the returned string.
 */
const xmlChar* xmlGetNodeName (xmlNodePtr cur) {
  if (cur == NULL) return (NULL);
  return cur->name;
  }

/**
 * xmlGetNodeNs:
 * @cur: the node pointer
 *
 * Returns pointer to node namespace URI or NULL.
 * The caller must not free the returned string.
 */
const xmlChar* xmlGetNodeNs (xmlNodePtr cur) {
  if (cur == NULL || cur->ns == NULL)
    return (NULL);
  return cur->ns->href;
  }

/**
 * xmlGetNodePrefix:
 * @cur: the node pointer
 *
 * Returns pointer to node namespace prefix or NULL.
 * The caller must not free the returned string.
 */
const xmlChar* xmlGetNodePrefix (xmlNodePtr cur) {
  if (cur == NULL || cur->ns == NULL)
    return (NULL);
  return cur->ns->prefix;
  }

/**
 * xmlGetPropType:
 * @cur: the attribute pointer
 *
 * Returns attribute type or 0 if attribute pointer is NULL.
 */
xmlAttrType xmlGetPropType (xmlAttrPtr cur) {
  if (cur == NULL) return 0;
  return cur->atype;
  }

/**
 * xmlGetPropName:
 * @cur: the attribute pointer
 *
 * Returns pointer to attribute name or NULL.
 * The caller must not free the returned string.
 */
const xmlChar* xmlGetPropName (xmlAttrPtr cur) {
  if (cur == NULL) return (NULL);
  return cur->name;
  }

/**
 * xmlGetPropNs:
 * @cur: the attribute pointer
 *
 * Returns pointer to attribute namespace URI or NULL.
 * The caller must not free the returned string.
 */
const xmlChar* xmlGetPropNs (xmlAttrPtr cur) {
  if (cur == NULL || cur->ns == NULL)
    return (NULL);
  return cur->ns->href;
  }

/**
 * xmlGetPropPrefix:
 * @cur: the attribute pointer
 *
 * Returns pointer to attribute namespace prefix or NULL.
 * The caller must not free the returned string.
 */
const xmlChar* xmlGetPropPrefix (xmlAttrPtr cur) {
  if (cur == NULL || cur->ns == NULL)
    return (NULL);
  return cur->ns->prefix;
  }

/**
 * xmlGetOwnerDoc:
 * @cur: the node pointer
 *
 * Returns pointer to containing document or NULL.
 */
xmlDocPtr xmlGetOwnerDoc (xmlNodePtr cur) {
  if (cur == NULL) return (NULL);
  return cur->doc;
  }

/**
 * xmlGetParent:
 * @cur: the node pointer
 *
 * Returns pointer to parent node or NULL.
 */
xmlNodePtr xmlGetParent (xmlNodePtr cur) {
  if (cur == NULL) return (NULL);
  return cur->parent;
  }

/**
 * xmlGetFirstChild:
 * @cur: the node pointer
 *
 * Returns pointer to first child node or NULL.
 */
xmlNodePtr xmlGetFirstChild (xmlNodePtr cur) {
  if (cur == NULL) return (NULL);
  return cur->children;
  }

/**
 * xmlGetPrevNode:
 * @cur: the node pointer
 *
 * Returns pointer to previous node or NULL.
 */
xmlNodePtr xmlGetPrevNode (xmlNodePtr cur) {
  if (cur == NULL) return (NULL);
  return cur->prev;
  }

/**
 * xmlGetNextNode:
 * @cur: the node pointer
 *
 * Returns pointer to next node or NULL.
 */
xmlNodePtr xmlGetNextNode (xmlNodePtr cur) {
  if (cur == NULL) return (NULL);
  return cur->next;
  }

/**
 * xmlGetFirstProp:
 * @cur: the node pointer
 *
 * Returns pointer to first attribute or NULL.
 */
xmlAttrPtr xmlGetFirstProp (xmlNodePtr cur) {
  if (cur == NULL) return (NULL);
  return cur->properties;
  }

/**
 * xmlGetPrevProp:
 * @cur: the attribute pointer
 *
 * Returns pointer to previous attribute or NULL.
 */
xmlAttrPtr xmlGetPrevProp (xmlAttrPtr cur) {
  if (cur == NULL) return (NULL);
  return cur->prev;
  }

/**
 * xmlGetNextProp:
 * @cur: the attribute pointer
 *
 * Returns pointer to next attribute or NULL.
 */
xmlAttrPtr xmlGetNextProp (xmlAttrPtr cur) {
  if (cur == NULL) return (NULL);
  return cur->next;
  }

/**
 * xmlGetEntityType:
 * @cur: the entity pointer
 *
 * Returns entity type or 0 if entity pointer is NULL.
 */
xmlEntityType xmlGetEntityType (xmlEntityPtr cur) {
  if (cur == NULL) return 0;
  return cur->etype;
  }

/**
 * xmlGetEntityName:
 * @cur: the entity pointer
 *
 * Returns pointer to entity name or NULL.
 * The caller must not free the returned string.
 */
const xmlChar* xmlGetEntityName (xmlEntityPtr cur) {
  if (cur == NULL) return (NULL);
  return cur->name;
  }

/**
 * xmlGetEntityExtID:
 * @cur: the entity pointer
 *
 * Returns pointer to entity external identifier or NULL.
 * The caller must not free the returned string.
 */
const xmlChar* xmlGetEntityExtID (xmlEntityPtr cur) {
  if (cur == NULL) return (NULL);
  return cur->ExternalID;
  }

/**
 * xmlGetEntitySysID:
 * @cur: the entity pointer
 *
 * Returns pointer to entity system identifier or NULL.
 * The caller must not free the returned string.
 */
const xmlChar* xmlGetEntitySysID (xmlEntityPtr cur) {
  if (cur == NULL) return (NULL);
  return cur->SystemID;
  }

/**
 * xmlRealloc:
 * @mem:  an already allocated block of memory
 * @size: the new size requested in bytes
 *
 * Returns a pointer to the newly reallocated block or NULL in case of error
 */
void* _xmlRealloc (void *mem, size_t size) {
  return xmlRealloc(mem,size);
  }

/**
 * _xmlMalloc:
 * @size:  the size requested in bytes
 *
 * Returns a pointer to the newly allocated block or NULL in case of error
 */
void* _xmlMalloc (size_t size) {
  return xmlMalloc(size);
  }

/**
 * _xmlFree:
 * @mem: an already allocated block of memory
 *
 * Free block of memory
 */
void _xmlFree (void *mem) {
  xmlFree(mem);
  return;
  }
